import { create } from 'zustand';
import { devtools } from 'zustand/middleware';
import type { GroupWithTasks, TaskWithSteps, StepWithNotes } from '../types';

interface TaskState {
  // Data
  groups: GroupWithTasks[];
  recentTasks: TaskWithSteps[];
  frequentTasks: TaskWithSteps[];
  currentStep: StepWithNotes | null;

  // Loading states
  isLoadingGroups: boolean;
  isLoadingRecentTasks: boolean;
  isLoadingFrequentTasks: boolean;
  isLoadingStep: boolean;

  // Selection state
  selectedGroup: GroupWithTasks | undefined;
  selectedTask: TaskWithSteps | undefined;
  selectedStep: StepWithNotes | undefined;

  // Actions
  setGroups: (groups: GroupWithTasks[]) => void;
  setRecentTasks: (tasks: TaskWithSteps[]) => void;
  setFrequentTasks: (tasks: TaskWithSteps[]) => void;
  setCurrentStep: (step: StepWithNotes | null) => void;

  setLoadingGroups: (loading: boolean) => void;
  setLoadingRecentTasks: (loading: boolean) => void;
  setLoadingFrequentTasks: (loading: boolean) => void;
  setLoadingStep: (loading: boolean) => void;

  setSelectedGroup: (group: GroupWithTasks | undefined) => void;
  setSelectedTask: (task: TaskWithSteps | undefined) => void;
  setSelectedStep: (step: StepWithNotes | undefined) => void;

  // Helper methods
  getTaskById: (taskId: number) => TaskWithSteps | undefined;
  getStepById: (stepId: number) => StepWithNotes | undefined;
  getGroupById: (groupId: number) => GroupWithTasks | undefined;

  // Update methods for mutations
  updateTaskPrintCount: (taskId: number, printCount: number, lastPrintedAt: string | null) => void;
  updateStepPrintCount: (stepId: number, printCount: number, lastPrintedAt: string | null) => void;
  addNoteToStep: (stepId: number, note: any) => void;
  addTaskToGroup: (groupId: number, task: TaskWithSteps) => void;
  addStepToTask: (taskId: number, step: StepWithNotes) => void;

  // Clear methods
  clearData: () => void;
}

export const useTaskStore = create<TaskState>()(
  devtools(
    (set, get) => ({
      // Initial state
      groups: [],
      recentTasks: [],
      frequentTasks: [],
      currentStep: null,

      isLoadingGroups: false,
      isLoadingRecentTasks: false,
      isLoadingFrequentTasks: false,
      isLoadingStep: false,

      selectedGroup: undefined,
      selectedTask: undefined,
      selectedStep: undefined,

      // Actions
      setGroups: (groups) => set({ groups }),
      setRecentTasks: (tasks) => set({ recentTasks: tasks }),
      setFrequentTasks: (tasks) => set({ frequentTasks: tasks }),
      setCurrentStep: (step) => set({ currentStep: step }),

      setLoadingGroups: (loading) => set({ isLoadingGroups: loading }),
      setLoadingRecentTasks: (loading) => set({ isLoadingRecentTasks: loading }),
      setLoadingFrequentTasks: (loading) => set({ isLoadingFrequentTasks: loading }),
      setLoadingStep: (loading) => set({ isLoadingStep: loading }),

      setSelectedGroup: (group) => set({ selectedGroup: group }),
      setSelectedTask: (task) => set({ selectedTask: task }),
      setSelectedStep: (step) => set({ selectedStep: step }),

      // Helper methods
      getTaskById: (taskId) => {
        const state = get();
        // Search in all groups
        for (const group of state.groups) {
          const task = group.tasks.find(t => t.id === taskId);
          if (task) return task;
        }
        // Search in recent tasks
        const recentTask = state.recentTasks.find(t => t.id === taskId);
        if (recentTask) return recentTask;
        // Search in frequent tasks
        const frequentTask = state.frequentTasks.find(t => t.id === taskId);
        if (frequentTask) return frequentTask;
        return undefined;
      },

      getStepById: (stepId) => {
        const state = get();
        // Search in all groups
        for (const group of state.groups) {
          for (const task of group.tasks) {
            const step = task.steps.find(s => s.id === stepId);
            if (step) return step;
          }
        }
        // Search in recent tasks
        for (const task of state.recentTasks) {
          const step = task.steps.find(s => s.id === stepId);
          if (step) return step;
        }
        // Search in frequent tasks
        for (const task of state.frequentTasks) {
          const step = task.steps.find(s => s.id === stepId);
          if (step) return step;
        }
        return undefined;
      },

      getGroupById: (groupId) => {
        const state = get();
        return state.groups.find(g => g.id === groupId);
      },

      // Update methods for mutations
      updateTaskPrintCount: (taskId, printCount, lastPrintedAt) => {
        set((state) => {
          const updateTaskInArray = (tasks: TaskWithSteps[]): TaskWithSteps[] =>
            tasks.map(task =>
              task.id === taskId
                ? {
                    ...task,
                    print_count: printCount,
                    last_printed_at: lastPrintedAt ? new Date(lastPrintedAt) : null,
                  }
                : task
            );

          return {
            groups: state.groups.map(group => ({
              ...group,
              tasks: updateTaskInArray(group.tasks),
            })),
            recentTasks: updateTaskInArray(state.recentTasks),
            frequentTasks: updateTaskInArray(state.frequentTasks),
          };
        });
      },

      updateStepPrintCount: (stepId, printCount, lastPrintedAt) => {
        set((state) => {
          const updateStepInTask = (task: TaskWithSteps): TaskWithSteps => ({
            ...task,
            steps: task.steps.map(step =>
              step.id === stepId
                ? {
                    ...step,
                    print_count: printCount,
                    last_printed_at: lastPrintedAt ? new Date(lastPrintedAt) : null,
                  }
                : step
            ),
          });

          const updateTaskInArray = (tasks: TaskWithSteps[]): TaskWithSteps[] =>
            tasks.map(task => updateStepInTask(task));

          return {
            groups: state.groups.map(group => ({
              ...group,
              tasks: updateTaskInArray(group.tasks),
            })),
            recentTasks: updateTaskInArray(state.recentTasks),
            frequentTasks: updateTaskInArray(state.frequentTasks),
          };
        });
      },

      addNoteToStep: (stepId, note) => {
        set((state) => {
          const addNoteToStepInTask = (task: TaskWithSteps): TaskWithSteps => ({
            ...task,
            steps: task.steps.map(step =>
              step.id === stepId
                ? {
                    ...step,
                    notes: [...step.notes, note],
                  }
                : step
            ),
          });

          const updateTaskInArray = (tasks: TaskWithSteps[]): TaskWithSteps[] =>
            tasks.map(task => addNoteToStepInTask(task));

          return {
            groups: state.groups.map(group => ({
              ...group,
              tasks: updateTaskInArray(group.tasks),
            })),
            recentTasks: updateTaskInArray(state.recentTasks),
            frequentTasks: updateTaskInArray(state.frequentTasks),
          };
        });
      },

      addTaskToGroup: (groupId, task) => {
        set((state) => ({
          groups: state.groups.map(group =>
            group.id === groupId
              ? { ...group, tasks: [...group.tasks, task] }
              : group
          ),
        }));
      },

      addStepToTask: (taskId, step) => {
        set((state) => {
          const addStepToTaskInArray = (tasks: TaskWithSteps[]): TaskWithSteps[] =>
            tasks.map(task =>
              task.id === taskId
                ? { ...task, steps: [...task.steps, step] }
                : task
            );

          return {
            groups: state.groups.map(group => ({
              ...group,
              tasks: addStepToTaskInArray(group.tasks),
            })),
            recentTasks: addStepToTaskInArray(state.recentTasks),
            frequentTasks: addStepToTaskInArray(state.frequentTasks),
          };
        });
      },

      clearData: () => {
        set({
          groups: [],
          recentTasks: [],
          frequentTasks: [],
          currentStep: null,
          selectedGroup: undefined,
          selectedTask: undefined,
          selectedStep: undefined,
        });
      },
    }),
    {
      name: 'task-store',
    }
  )
);