import { Knex } from 'knex';
import { GroupRepository, TaskRepository, StepRepository, UserRepository, NoteRepository, PrintHistoryRepository, ImageRepository } from '../db/repositories';
import { printTask, printStep } from '../printer/helpers';
import type { Printer, Group, Task, Step, User, Note, PrintHistory, Image } from '@shared/index';

interface Context {
  db: Knex;
  printer: Printer;
}

// GraphQL resolver types
type ResolverParent = unknown;
type ResolverArgs = Record<string, unknown>;
type ResolverResult<T> = Promise<T | null>;

export const resolvers = {
  Query: {
    groups: async (_parent: ResolverParent, _args: ResolverArgs, { db }: Context): ResolverResult<Group[]> => {
      const groupRepo = new GroupRepository(db);
      return await groupRepo.findAll();
    },
    group: async (_parent: ResolverParent, { id }: { id: string }, { db }: Context): ResolverResult<Group> => {
      const groupRepo = new GroupRepository(db);
      return await groupRepo.findById(parseInt(id));
    },
    tasks: async (_parent: ResolverParent, { groupId }: { groupId: string }, { db }: Context): ResolverResult<Task[]> => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findByGroupId(parseInt(groupId));
    },
    task: async (_parent: ResolverParent, { id }: { id: string }, { db }: Context): ResolverResult<Task> => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findById(parseInt(id));
    },
    steps: async (_parent: ResolverParent, { taskId }: { taskId: string }, { db }: Context): ResolverResult<Step[]> => {
      const stepRepo = new StepRepository(db);
      return await stepRepo.findByTaskId(parseInt(taskId));
    },
    step: async (_parent: ResolverParent, { id }: { id: string }, { db }: Context): ResolverResult<Step> => {
      const stepRepo = new StepRepository(db);
      return await stepRepo.findById(parseInt(id));
    },
    recentTasks: async (_parent: ResolverParent, _args: ResolverArgs, { db }: Context): ResolverResult<Task[]> => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findRecent();
    },
    frequentTasks: async (_parent: ResolverParent, _args: ResolverArgs, { db }: Context): ResolverResult<Task[]> => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findFrequent();
    },
    users: async (_parent: ResolverParent, _args: ResolverArgs, { db }: Context): ResolverResult<User[]> => {
      const userRepo = new UserRepository(db);
      return await userRepo.findAll();
    },
    user: async (_parent: ResolverParent, { id }: { id: string }, { db }: Context): ResolverResult<User> => {
      const userRepo = new UserRepository(db);
      return await userRepo.findById(parseInt(id));
    },
    notes: async (_parent: ResolverParent, { stepId, taskId }: { stepId?: string; taskId?: string }, { db }: Context): ResolverResult<Note[]> => {
      const noteRepo = new NoteRepository(db);
      if (stepId) {
        return await noteRepo.findByStepId(parseInt(stepId));
      }
      if (taskId) {
        return await noteRepo.findByTaskId(parseInt(taskId));
      }
      return await noteRepo.findAll();
    },
    printHistory: async (_parent: ResolverParent, { taskId, stepId }: { taskId?: string; stepId?: string }, { db }: Context): ResolverResult<PrintHistory[]> => {
      const printHistoryRepo = new PrintHistoryRepository(db);
      if (taskId) {
        return await printHistoryRepo.findByTaskId(parseInt(taskId));
      }
      if (stepId) {
        return await printHistoryRepo.findByStepId(parseInt(stepId));
      }
      return await printHistoryRepo.findAll();
    },
  },

  Mutation: {
    createGroup: async (_parent: ResolverParent, { name, parentId }: { name: string; parentId?: string }, { db }: Context): ResolverResult<Group> => {
      const groupRepo = new GroupRepository(db);
      let parent_id: number | undefined = undefined;
      if (parentId) {
        const parent = await groupRepo.findById(parseInt(parentId));
        if (!parent) throw new Error('Parent group not found');
        parent_id = parseInt(parentId);
      }
      return await groupRepo.create({ name, parent_id });
    },
    createTask: async (_parent: ResolverParent, { name, groupId }: { name: string; groupId: string }, { db }: Context): ResolverResult<Task> => {
      const groupRepo = new GroupRepository(db);
      const taskRepo = new TaskRepository(db);
      const group = await groupRepo.findById(parseInt(groupId));
      if (!group) throw new Error('Group not found');
      return await taskRepo.create({
        name,
        group_id: parseInt(groupId),
        print_count: 0,
      });
    },
    createStep: async (_parent: ResolverParent, { name, instructions, taskId, order }: { name: string; instructions: string; taskId: string; order: number }, { db }: Context): ResolverResult<Step> => {
      const taskRepo = new TaskRepository(db);
      const stepRepo = new StepRepository(db);
      const task = await taskRepo.findById(parseInt(taskId));
      if (!task) throw new Error('Task not found');
      return await stepRepo.create({
        name,
        instructions,
        task_id: parseInt(taskId),
        order,
        print_count: 0,
      });
    },
    createImage: async (_parent: ResolverParent, { stepId, originalPath, bwPath, order }: { stepId: string; originalPath: string; bwPath: string; order: number }, { db }: Context): ResolverResult<Image> => {
      const stepRepo = new StepRepository(db);
      const imageRepo = new ImageRepository(db);
      const step = await stepRepo.findById(parseInt(stepId));
      if (!step) throw new Error('Step not found');
      return await imageRepo.create({
        step_id: parseInt(stepId),
        original_path: originalPath,
        bw_path: bwPath,
        order,
      });
    },
    createUser: async (_parent: ResolverParent, { name }: { name: string }, { db }: Context): ResolverResult<User> => {
      const userRepo = new UserRepository(db);
      return await userRepo.create({ name });
    },
    createNote: async (
      _parent: ResolverParent,
      { content, stepId, taskId, userId }: { content: string; stepId?: string; taskId?: string; userId: string },
      { db }: Context
    ): ResolverResult<Note> => {
      const userRepo = new UserRepository(db);
      const stepRepo = new StepRepository(db);
      const taskRepo = new TaskRepository(db);
      const noteRepo = new NoteRepository(db);
      const user = await userRepo.findById(parseInt(userId));
      if (!user) throw new Error('User not found');
      if (stepId) {
        const step = await stepRepo.findById(parseInt(stepId));
        if (!step) throw new Error('Step not found');
      } else if (taskId) {
        const task = await taskRepo.findById(parseInt(taskId));
        if (!task) throw new Error('Task not found');
      }
      return await noteRepo.create({
        content,
        step_id: stepId ? parseInt(stepId) : undefined,
        task_id: taskId ? parseInt(taskId) : undefined,
        created_by: parseInt(userId),
      });
    },
    printTask: async (_parent: ResolverParent, { id, userId }: { id: string; userId: string }, { db, printer }: Context): ResolverResult<Task> => {
      const userRepo = new UserRepository(db);
      const user = await userRepo.findById(parseInt(userId));
      if (!user) throw new Error('User not found');
      return await printTask(parseInt(id), parseInt(userId), db, printer);
    },
    printStep: async (_parent: ResolverParent, { id, userId }: { id: string; userId: string }, { db, printer }: Context): ResolverResult<Step> => {
      const userRepo = new UserRepository(db);
      const user = await userRepo.findById(parseInt(userId));
      if (!user) throw new Error('User not found');
      return await printStep(parseInt(id), parseInt(userId), db, printer);
    },
  },

  Group: {
    tasks: async (group: { id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Task[]> => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findByGroupId(group.id);
    },
  },

  Task: {
    group: async (task: { group_id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Group> => {
      const groupRepo = new GroupRepository(db);
      return await groupRepo.findById(task.group_id);
    },
    steps: async (task: { id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Step[]> => {
      const stepRepo = new StepRepository(db);
      return await stepRepo.findByTaskId(task.id);
    },
    notes: async (task: { id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Note[]> => {
      const noteRepo = new NoteRepository(db);
      return await noteRepo.findByTaskId(task.id);
    },
    printHistory: async (task: { id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<PrintHistory[]> => {
      const printHistoryRepo = new PrintHistoryRepository(db);
      return await printHistoryRepo.findByTaskId(task.id);
    },
  },

  Step: {
    task: async (step: { task_id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Task> => {
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findById(step.task_id);
    },
    images: async (step: { id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Image[]> => {
      const imageRepo = new ImageRepository(db);
      return await imageRepo.findByStepId(step.id);
    },
    notes: async (step: { id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Note[]> => {
      const noteRepo = new NoteRepository(db);
      return await noteRepo.findByStepId(step.id);
    },
    printHistory: async (step: { id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<PrintHistory[]> => {
      const printHistoryRepo = new PrintHistoryRepository(db);
      return await printHistoryRepo.findByStepId(step.id);
    },
  },

  Note: {
    createdBy: async (note: { created_by: number }, _args: ResolverArgs, { db }: Context): ResolverResult<User> => {
      const userRepo = new UserRepository(db);
      return await userRepo.findById(note.created_by);
    },
  },

  PrintHistory: {
    user: async (history: { user_id: number }, _args: ResolverArgs, { db }: Context): ResolverResult<User> => {
      const userRepo = new UserRepository(db);
      return await userRepo.findById(history.user_id);
    },
    task: async (history: { task_id?: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Task> => {
      if (!history.task_id) return null;
      const taskRepo = new TaskRepository(db);
      return await taskRepo.findById(history.task_id);
    },
    step: async (history: { step_id?: number }, _args: ResolverArgs, { db }: Context): ResolverResult<Step> => {
      if (!history.step_id) return null;
      const stepRepo = new StepRepository(db);
      return await stepRepo.findById(history.step_id);
    },
  },
};