import { PAPER_CONFIG } from "./printer-constants";

export const formatUtils = {
  /**
   * Creates a banner line with the specified character
   * @param char Character to repeat
   * @param length Length of the banner
   * @returns Formatted banner string
   */
  createBanner(char: string, length: number = PAPER_CONFIG.BANNER_LENGTH): string {
    return char.repeat(length);
  },

  /**
   * Formats a checkbox item with the given text
   * @param text Text to display after the checkbox
   * @returns Formatted checkbox string
   */
  formatCheckbox(text: string): string {
    return `[ ] ${text}`;
  },

  /**
   * Formats a list of items into lines with optional numbering
   * @param items List of items to format
   * @param startIndex Starting index for numbering (0 for no numbers)
   * @returns Array of formatted lines
   */
  formatList(items: string[], startIndex: number = 0, prefix: string = ''): string[] {
    return items.map((item, index) => {
      const num = startIndex > 0 ? `${index + startIndex}: ` : '';
      const body = this.formatCheckbox(`${num}${item}`);
      return `${prefix}${body}`;
    });
  },

  /**
   * Formats a step header with task context
   * @param stepName Name of the step
   * @param stepNumber Step number (1-based)
   * @param taskName Name of the parent task (only used in single step view)
   * @param isTaskView Whether this is being used in a task view
   * @returns Formatted step header
   */
  formatStepHeader(stepName: string, stepNumber: number, taskName?: string, isTaskView: boolean = false): string {
    const parts = ['Step'];
    if (stepNumber !== undefined) {
      parts.push(' ');
      parts.push(stepNumber.toString());
    }
    if (!isTaskView && taskName) {
      parts.push(` of ${taskName}`);
    }
    parts.push(': ');
    parts.push(stepName);
    return parts.join('');
  },

  /**
   * Formats a section with a header and content
   * @param header Section header
   * @param content Section content
   * @param bannerChar Character to use for the banner
   * @returns Array of formatted lines
   */
  formatSection(header: string, content: string, bannerChar: string = '=', trailingNewline: boolean = false): string[] {
    const parts = [
      this.formatCheckbox(header),
      this.createBanner(bannerChar),
      content,
    ];
    if (trailingNewline) {
      parts.push('');
    }
    return parts;
  }
};